;;; Copyright (C) 1995 Sho-Huan Simon Tung
;;; See ./VES2/COPYING for details

(case-sensitive #t)
(current-expand eps-expand)

(define rllr-lambda-pos
  (lambda (exp env tail fno)
    (make-exp-env (car exp) env 
		  (if (not (symbol? (car exp))) ; this symbol must eval
		      #f                        ; to a proc or closure.
		      tail)
		  fno)))

(define rllr-map
  (lambda (f l)
    (if (null? l)
	'()
	(let ([dr (rllr-map f (cdr l))])
	  (cons (f (car l)) dr)))))

(define tail-enable? (lambda () #t))

(define current-exp-env #f)
(define display-env? #t)

(define debug-flag #f)
(define steps 0)
(define run-till '())
(define break-on-call/cc #f)
(define micro #f)
(define cadddddr (lambda (l) (car (cddr (cdddr l)))))

(define call-cc-arg #f)
(define call-cc-called #f)
(define new-control #f)
(define cont-called #f)

(define-structure (exp-env exp env tail fno))

(define env-pt caddr)
(define body cddr)
(define formals cadr)

(define cont-val (make-exp-env '(!V) 'to-receive #f 0))

(define value?
  (lambda (env)
    (or (eq? 'after-lookup env)
	(eq? 'after-apply env)   ; appears after apply system procedures only.
	(eq? 'after-set! env)
	(eq? 'after-lambda env)
	(eq? 'simple-value env)
	(eq? 'to-receive env)
	(eq? 'cont-obj env))))

(define closure? (lambda (p) (and (pair? p) (eq? '*closure* (car p)))))
(define continuation? (lambda (p) (and (pair? p) (eq? '!K (car p)))))

;(load "/tungsh/Apps/RainbowScheme/VES2/tracers/src/repl.ss")
;(load "/tungsh/Apps/RainbowScheme/VES2/tracers/src/display.ss")
;(load "/tungsh/Apps/RainbowScheme/VES2/tracers/src/gc.ss")
;(load "/tungsh/Apps/RainbowScheme/VES2/tracers/src/eval.ss")
(load "/mis/home/faculty/tungsh/trace-serv/VES2/tracers/load.ss")

(if (not (top-level-bound? '**saved**)) (set! old-apply apply))
(define apply
  (lambda (f . l)
    (let ([arg ((rec loop
		  (lambda (l)
		    (if (null? l)
			'()
			(if (null? (cdr l))
			    (car l)
			    (cons (car l) (loop (cdr l))))))) l)])
      (if (procedure? f)
	  (old-apply f arg)
	  (if (closure? f)
	      (let ([e `(,(make-exp-env f 'after-lookup #f 0)
			    ,@(map (lambda (e)
				     (make-exp-env e 'simple-value #f 0))
				   arg))])
		  e)
	      (error 'apply "Not a procedure or closure ~s" f))))))

(if (not (top-level-bound? '**saved**)) (set! old-call/cc call/cc))
(define call/cc
  (lambda (p)
    (if (procedure? p)
	(old-call/cc p)
	(if (closure? p)
	    (begin
	      (set! steps 0)
	      (set! call-cc-arg
		    (make-exp-env (cadr p) (caddr p) #f 0))
	      (set! call-cc-called #t)
	      cont-val)
	    (error 'call/cc "Not a procedure or closure ~s" p)))))
(define call-with-current-continuation call/cc)

(if (not (top-level-bound? '**saved**)) (set! old-map map))
(define map
  (lambda (f ls . more)
    (if (procedure? f)
	(apply old-map f (cons ls more))
	(let ([do-map
		(lambda (f ls more)
		  (if (null? more)
		      ((rec map1 (lambda (ls)
				   (if (null? ls)
				       '()
				       (cons
					 (list
					   (make-exp-env f 'after-lookup #f 0)
					   (make-exp-env
					     (car ls)
					     'simple-value
					     #f 0))
					 (map1 (cdr ls))))))
		       ls)
		      (let map-more ([ls ls] [more more])
			(if (or (null? ls)
				(ormap null? more))
			    '()
			    (cons `(,(make-exp-env f 'after-lookup #f 0)
				     ,(make-exp-env
					(car ls)
					'simple-value
					#f
					0)
				     ,@(old-map
					 (lambda (e)
					   (make-exp-env
					     e
					     'simple-value
					     #f 0))
					 (old-map car more)))
				  (map-more (cdr ls)
				    (old-map cdr more)))))))])
	  `(,(make-exp-env 'list
	       (list (get-top-level)) #f 0)
	    ,@(do-map f ls more))))))

(if (not (top-level-bound? '**saved**)) (set! old-for-each for-each))
(define for-each
  (lambda (f ls . more)
    (if (procedure? f)
	(apply old-for-each f (cons ls more))
	(let ([do-map
		(lambda (f ls more)
		  (if (null? more)
		      ((rec map1 (lambda (ls)
				   (if (null? ls)
				       '()
				       (cons
					 (list
					   (make-exp-env f 'after-lookup #f 0)
					   (make-exp-env
					     (car ls)
					     'simple-value
					     #f 0))
					 (map1 (cdr ls))))))
		       ls)
		      (let map-more ([ls ls] [more more])
			(if (or (null? ls)
				(ormap null? more))
			    '()
			    (cons `(,(make-exp-env f 'after-lookup #f 0)
				     ,(make-exp-env
					(car ls)
					'simple-value
					#f
					0)
				     ,@(old-map
					 (lambda (e)
					   (make-exp-env
					     e
					     'simple-value
					     #f 0))
					 (old-map car more)))
				  (map-more (cdr ls)
				    (old-map cdr more)))))))])
	  `(*this*is*for*each*
	    ,@(do-map f ls more))))))

(define FD
  (lambda (n)
    (display (list '!G (list 'FD n)))
    (newline)))

(define BK
  (lambda (n)
    (display (list '!G (list 'BK n)))
    (newline)))

(define RT
  (lambda (n)
    (display (list '!G (list 'RT n)))
    (newline)))

(define LT
  (lambda (n)
    (display (list '!G (list 'LT n)))
    (newline)))

(define CS
  (lambda ()
    (display (list '!G '(CS)))
    (newline)))

(define HOME
  (lambda ()
    (display (list '!G '(HOME)))
    (newline)))

(define PD
  (lambda ()
    (display (list '!G '(PD)))
    (newline)))

(define PU
  (lambda ()
    (display (list '!G '(PU)))
    (newline)))


(define **saved** #t)
(define main
  (lambda ()
    (env-init)
    (newline)
    (ves #f)))

(main)

