;;; Copyright (C) 1995 Sho-Huan Simon Tung
;;; See ./VES2/COPYING for details

(define kill-time
  (lambda (m)     ; slow down the output for front-end to consume.
    ((rec loop
	  (lambda (n)
	    (if (= n 0)
		#f
		(loop (- n 1))))) m)))

(define pair-for-each
  (lambda (proc ls)
    (if (not (null? ls))
	(if (pair? ls)
	    (begin
	      (proc (car ls))
	      (pair-for-each proc (cdr ls)))
	    (begin
	      (display " ")
	      (display ".")
	      (proc ls))))))
	
(define print-frame-no
  (lambda (env)
    (if (null? env)
	'()
	(cons (caar env) (print-frame-no (cdr env))))))

(define output-mgr
  (lambda (rsl)
    (let ([output (output-mgr-h rsl)])
      output)))

(define output-mgr-h
  (lambda (rsl)
    (if (not debug-flag)
	(if (pair? rsl)
	    (case (car rsl)
	      [(set!)
	       (if (exp-env? (cadr rsl))
		   (let ([id (cadr rsl)]
			 [v  (caddr rsl)])
		     `(set! ,(exp-env-exp id) 
			,(output-mgr-h v)))
		   (pairmap output-mgr-h rsl))]
	      [(*closure*)
	       (let ([lam (cadr rsl)])
		 `(!L ,(cadr lam) ,(caddr lam) ,(print-frame-no (caddr rsl))))]
	      [else (pairmap output-mgr-h rsl)])
	    (if (exp-env? rsl)
		(let ([exp (exp-env-exp rsl)]
		      [env (exp-env-env rsl)])
		  (if (value? env)
		      (if (pair? exp)
			  (if (eq? '*closure* (car exp))
			      (let ([lam (cadr exp)])
				`(!L ,(cadr lam) ,(caddr lam) ,(print-frame-no
								(caddr exp))))
			      (if (eq? 'cont-obj env)
				  `(!K ,(output-mgr-h (cadr exp)))
				  (pairmap output-mgr-h exp)))
			  (if (procedure? exp)
				  (let ([p (open-output-string)])
				    (write exp p)
				    (let* ([s (get-output-string p)]
					   [l (string-length s)]
					   [p-or-sp (substring s 2 3)])
				      (if (string=? p-or-sp "p")
					  (string->symbol
					    (substring s 12 (- l 1)))
					  (string->symbol
					    (substring s 19 (- l 1))))))
				  exp))
		      (if (and (pair? exp) (not (eq? '*closure* (car exp))))
			  `(!E ,(pairmap output-mgr-h exp) 
			     ,(print-frame-no env))
			  `(!E ,exp  ,(print-frame-no env)))))
		    rsl))
	rsl)))

(define pairmap
  (lambda (f p)
    (if (null? p)
	'()
	(if (pair? (cdr p))
	    (cons (f (car p)) (pairmap f (cdr p)))
	    (if (null? (cdr p))
		(list (f (car p)))
		(cons (f (car p)) (f (cdr p))))))))

(define pair+
  (lambda (p)
    (if (null? p)
	0
	(if (pair? (cdr p))
	    (+ (car p) (pair+ (cdr p)))
	    (if (null? (cdr p))
		(car p)
		(+ (car p) (cdr p)))))))

(define my-pretty
  (lambda (l)
    (if (pair? l)
	(let ([len  (pair+ (pairmap leng l))]
	      [leng (let* ([p (open-output-string)]
			   [ig (write l p)])
		      (string-length (get-output-string p)))])
	  (if (> len 40)
	      (begin
		(pretty-print l)
		(kill-time (* 20 leng)))
	      (begin
		(display "(")
		(write (car l))
		(pair-for-each
		  (lambda (o)
		    (display " ")
		    (write o))
		  (cdr l))
		(display ")")
		(newline)
		(kill-time (* 20 leng)))))
	(pretty-print l))))
	      
(define leng
  (lambda (l)
    (if (pair? l)
	(case (car l)
	  ((!L !K) 2)
	  ((!E) (let ([exp (cadr l)])
		  (let ([p (open-output-string)])
		    (write exp p)
		    (string-length (get-output-string p)))))
	  (else	(pair+ (pairmap leng l))))
	(let ([p (open-output-string)])
		  (write l p)
		  (string-length (get-output-string p))))))

(define pretty-env
  (lambda (l)
      (cond
       ((pair? l) (if (eq? '!L (car l))
		      (let* ([p (open-output-string)]
			     [ignore (write l p)]
			     [leng (string-length (get-output-string p))])
			(display "(!L ")
			(write (cadr l)) (newline)
			(pretty-print (caddr l))
			(write (cadddr l))
			(display ")")
			(kill-time (* 20 leng)))
		     (if (eq? '!K (car l))
			 (let* ([p (open-output-string)]
				[ignore (write l p)]
				[leng (string-length (get-output-string p))])
			   (display "(!K ")
			   (newline)
			   (pretty-print (cadr l))
			   (display ")")
			   (kill-time (* 20 leng)))
			 (begin
			   (display "(")
			   (pretty-env (car l))
			   (pair-for-each
			     (lambda (x)
			       (display " ")
			       (pretty-env x)) (cdr l))
			   (display ")")))))
      (else (write l)))))

